from Products.ATContentTypes.content.image import ATImage
from Products.Archetypes.public import registerType
from Products.SimpleAttachment.config import PROJECTNAME

class ImageAttachment(ATImage):
    """An image attachment"""
    portal_type = meta_type = 'ImageAttachment'


registerType(ImageAttachment, PROJECTNAME)



from Products.SimpleAttachment.tests.base import IntegrationTestCase


class TestInstallation(IntegrationTestCase):
    """Ensure product is properly installed"""

    def afterSetUp(self):
        self.css        = self.portal.portal_css
        self.kupu       = self.portal.kupu_library_tool
        self.skins      = self.portal.portal_skins
        self.types      = self.portal.portal_types
        self.factory    = self.portal.portal_factory
        self.workflow   = self.portal.portal_workflow
        self.properties = self.portal.portal_properties

        self.metaTypes = ('ImageAttachment', 'FileAttachment')

    def testSkinLayersInstalled(self):
        self.failUnless('attachment_widgets' in self.skins.objectIds())
        self.failUnless('simpleattachment' in self.skins.objectIds())

    def testTypesInstalled(self):
        for t in self.metaTypes:
            self.failUnless(t in self.types.objectIds())

    def testTypesNotSearched(self):
        types_not_searched = self.properties.site_properties.getProperty('types_not_searched')
        self.failUnless('FileAttachment' in types_not_searched)
        self.failUnless('ImageAttachment' in types_not_searched)

    def testTypesUseViewActionInListings(self):
        typesUseViewActionInListings = self.properties.site_properties.getProperty('typesUseViewActionInListings')
        self.failUnless('FileAttachment' in typesUseViewActionInListings)
        self.failUnless('ImageAttachment' in typesUseViewActionInListings)

    def testAttachmentsHaveNoWorkflow(self):
        self.assertEqual(self.workflow.getChainForPortalType('FileAttachment'), ())
        self.assertEqual(self.workflow.getChainForPortalType('ImageAttachment'), ())

    def testKupuResources(self):
        linkable = self.kupu.getPortalTypesForResourceType('linkable')
        mediaobject = self.kupu.getPortalTypesForResourceType('mediaobject')
        self.failUnless('FileAttachment' in linkable)
        self.failUnless('ImageAttachment' in linkable)
        self.failUnless('ImageAttachment' in mediaobject)


def test_suite():
    from unittest import defaultTestLoader
    return defaultTestLoader.loadTestsFromName(__name__)



from Products.PloneTestCase import ptc
from Products.Five import zcml, fiveconfigure
from Products.Five.testbrowser import Browser
from collective.testcaselayer.ptc import BasePTCLayer, ptc_layer


ptc.installProduct('SimpleAttachment', quiet=True)
ptc.installProduct('RichDocument', quiet=True)
ptc.setupPloneSite(products=['SimpleAttachment', 'RichDocument'])


class Layer(BasePTCLayer):
    """ basic layer for testing """

    def afterSetUp(self):
        # load zcml for this package...
        fiveconfigure.debug_mode = True
        from Products import SimpleAttachment
        zcml.load_config('configure.zcml', package=SimpleAttachment)
        fiveconfigure.debug_mode = False

layer = Layer(bases=[ptc_layer])


class IntegrationTestCase(ptc.PloneTestCase):
    """ base class for integration tests """

    layer = layer


class FunctionalTestCase(ptc.FunctionalTestCase):
    """ base class for functional tests """

    layer = layer

    def getBrowser(self, loggedIn=True):
        """ instantiate and return a testbrowser for convenience """
        browser = Browser()
        if loggedIn:
            user = ptc.default_user
            pwd = ptc.default_password
            browser.addHeader('Authorization', 'Basic %s:%s' % (user, pwd))
        return browser



from copy import deepcopy
from logging import getLogger
from zlib import compress, decompress
from zope.annotation.interfaces import IAnnotations
from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View
from Products.ATContentTypes.content.file import ATFile
from Products.Archetypes.public import registerType, FileField
from Products.SimpleAttachment.config import PROJECTNAME

debug = getLogger(__name__).debug
key = 'transforms_cache'


class CachingFileField(FileField):
    """ extended version of AT's file-field support a transforms caching """

    def set(self, instance, value, **kwargs):
        annotations = IAnnotations(instance)
        if key in annotations:
            del annotations[key]
        return super(CachingFileField, self).set(instance, value, **kwargs)


def makeField(field):
    """ mostly copied from `Archetypes.Field.Field.copy` """
    cdict = dict(vars(field))
    cdict.pop('__name__')
    # Widget must be copied separatedly
    widget = cdict['widget']
    del cdict['widget']
    properties = deepcopy(cdict)
    properties['widget'] = widget.copy()
    return CachingFileField(field.getName(), **properties)


schema = ATFile.schema.copy()
schema['file'] = makeField(schema['file'])
schema['file'].index_method = 'indexData'


class FileAttachment(ATFile):
    """A file attachment"""

    portal_type = meta_type = 'FileAttachment'
    schema = schema
    security = ClassSecurityInfo()

    security.declareProtected(View, 'indexData')
    def indexData(self, mimetype=None):
        """ index accessor with caching of the result """
        if not mimetype == 'text/plain':
            return self.getFile()
        annotations = IAnnotations(self)
        if key in annotations:
            debug('using cached transforms value for %r', self)
            value = decompress(annotations[key])
        else:
            debug('getting transforms value for %r', self)
            value = self.getField('file').getIndexable(self)
            if value:
                annotations[key] = compress(value)
        return value

    def setFile(self, value):
        """ wrapper for "file" field mutator with cache invalidation """
        annotations = IAnnotations(self)
        if key in annotations:
            del annotations[key]
        return super(FileAttachment, self).setFile(value)


registerType(FileAttachment, PROJECTNAME)



from os.path import dirname, join


def getFile(filename):
    """ return a file object from the test folder """
    from Products.SimpleAttachment import tests
    filename = join(dirname(tests.__file__), filename)
    return open(filename, 'r')


def getData(filename):
    """ return file data """
    return getFile(filename).read()



from Products.CMFCore.permissions import setDefaultRoles

PROJECTNAME = "SimpleAttachment"
GLOBALS = globals()
DEFAULT_ADD_CONTENT_PERMISSION = "Add Attachment"

setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner',))



from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget


class ImagesManagerWidget(TypesWidget):
    """This widget adds support for uploading images into documents. To support
    this, you must use it on a folderish type (derived from BaseFolder) with
    'ImageAttachment' in the allowed_content_types. Create a BooleanField and
    use this widget. This will display a form at the bottom of your base_edit
    (presuming it's the last widget, which it probably ought to be) where you
    can upload images into your content type. The boolean field itself is
    used to select whether an image download box should be presented. This
    is similar to the "related items" box in Plone.

    Content editors may also reference the images directly in their body
    text.

    Caveats: In the edit macro, the upload button may steal the default
    enter-key-press in base_edit.
    """

    # Use the base class properties, and add two of our own
    _properties = TypesWidget._properties.copy()
    _properties.update({'macro'     : 'widget_imagesmanager',
                        'expanded'  : False,
                        },)


# Register the widget with Archetypes
registerWidget(ImagesManagerWidget,
               title = 'Images manager widget',
               description= ('Renders controls for uploading images in documents',),
               used_for = ('Products.Archetypes.Field.BooleanField',)
               )



from Products.Archetypes.atapi import process_types, listTypes
from Products.CMFCore import utils, DirectoryView
from Products.SimpleAttachment.config import PROJECTNAME
from Products.SimpleAttachment.config import DEFAULT_ADD_CONTENT_PERMISSION


DirectoryView.registerDirectory('skins', globals())


def initialize(context):

    # Import the type, which results in registerType() being called
    from content import FileAttachment, ImageAttachment
    FileAttachment, ImageAttachment     # make pyflakes happy :)

    # initialize the content, including types and add permissions
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = DEFAULT_ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)



from Products.CMFPlone.utils import _createObjectByType as create
from Products.SimpleAttachment.tests.base import IntegrationTestCase
from Products.SimpleAttachment.tests.utils import getData


class CachingTests(IntegrationTestCase):

    def afterSetUp(self):
        self.setRoles(('Manager',))

    def makeOne(self, filename):
        return create('FileAttachment', self.folder, 'foo',
            title='Foo Bar', file=getData(filename))

    def testSearchableTextReturnsText(self):
        attachment = self.makeOne('plone3.pdf')
        self.failUnless('Plone 3' in attachment.SearchableText())

    def testSearchableTextCachesTransformedFile(self):
        attachment = self.makeOne('plone3.pdf')
        original = attachment.SearchableText()
        # after the first call the transformed output should be cached
        # so let's sneak in a different value and check again...
        storage = attachment.getField('file').getStorage()
        storage.set('file', attachment, 'hurz!')
        self.assertEqual(attachment.getFile().data, 'hurz!')
        self.assertEqual(attachment.SearchableText(), original)

    def testMutatorInvalidatesCache(self):
        attachment = self.makeOne('plone3.pdf')
        self.failUnless('Plone 3' in attachment.SearchableText())
        attachment.setFile(getData('plone4.pdf'))
        self.failUnless('Plone 4' in attachment.SearchableText())

    def testFieldSetInvalidatesCache(self):
        attachment = self.makeOne('plone3.pdf')
        self.failUnless('Plone 3' in attachment.SearchableText())
        attachment.getField('file').set(attachment, getData('plone4.pdf'))
        self.failUnless('Plone 4' in attachment.SearchableText())

    def testUpdateInvalidatesCache(self):
        attachment = self.makeOne('plone3.pdf')
        self.failUnless('Plone 3' in attachment.SearchableText())
        attachment.update(file=getData('plone4.pdf'))
        self.failUnless('Plone 4' in attachment.SearchableText())


def test_suite():
    from unittest import defaultTestLoader
    return defaultTestLoader.loadTestsFromName(__name__)



from setuptools import setup, find_packages
import os

version = '3.3'

setup(name='Products.SimpleAttachment',
      version=version,
      description="Simple Attachments for Plone",
      long_description=open("README.txt").read() + '\n' + open('docs/HISTORY.txt').read(),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='Plone attachments RichDocument',
      author='Martin Aspeli',
      author_email='',
      url='',
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
      ],
      extras_require = { 'test': [
          'zope.testing',
          'collective.testcaselayer',
          'Products.RichDocument',
      ]},
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



"""
    SimpleAttachment functional doctests.  This module collects all *.txt
    files in the tests directory and runs them.

    Based partly on test_functional.py from CMFPlone and the test tutorial:
    http://plone.org/documentation/tutorial/testing/doctests

"""
import os
import glob
import doctest
import unittest
from Globals import package_home
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite

from Products.SimpleAttachment.tests import GLOBALS
from Products.SimpleAttachment.tests.base import FunctionalTestCase

OPTIONFLAGS = (
    doctest.ELLIPSIS |
    doctest.NORMALIZE_WHITESPACE
)


def list_doctests():
    home = package_home(GLOBALS)
    return [filename for filename in
          glob.glob(os.path.sep.join([home, '*.txt']))]


def test_suite():
    filenames = list_doctests()
    suites = [Suite(os.path.basename(filename),
               optionflags=OPTIONFLAGS,
               package='Products.SimpleAttachment.tests',
               test_class=FunctionalTestCase)
              for filename in filenames]
    return unittest.TestSuite(suites)



from Products.CMFCore.utils import getToolByName

def setupAttachments(context):
    if context.readDataFile('simpleattachment_various.txt') is None:
        return

    portal = context.getSite()

    # Add FileAttachment and ImageAttachment to kupu's linkable and media types
    kupuTool = getToolByName(portal, 'kupu_library_tool', None)
    tinyTool = getToolByName(portal, 'portal_tinymce', None)
    if kupuTool:
        linkable = list(kupuTool.getPortalTypesForResourceType('linkable'))
        mediaobject = list(kupuTool.getPortalTypesForResourceType('mediaobject'))
        if 'FileAttachment' not in linkable:
            linkable.append('FileAttachment')
        if 'ImageAttachment' not in linkable:
            linkable.append('ImageAttachment')
        if 'ImageAttachment' not in mediaobject:
            mediaobject.append('ImageAttachment')
        # kupu_library_tool has an idiotic interface, basically written purely to
        # work with its configuration page. :-(
        kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                       'old_type'      : 'linkable',
                                       'portal_types'  :  linkable},
                                      {'resource_type' : 'mediaobject',
                                       'old_type'      : 'mediaobject',
                                       'portal_types'  :  mediaobject},))
                                
    if tinyTool:
        linkable = tinyTool.linkable.split('\n')
        linkable.append('ImageAttachment')
        linkable.append('FileAttachment')
        tinyTool.linkable = '\n'.join(set(linkable))
        
        imageobjects = tinyTool.imageobjects.split('\n')
        imageobjects.append('ImageAttachment')
        tinyTool.imageobjects = '\n'.join(set(imageobjects))
        


def registerImagesFormControllerActions(context, contentType=None, template='base_edit'):
    """Register the form controller actions necessary for the widget to work.
    This should probably be called from the Install.py script. The parameter
    'context' should be the portal root or another place from which the form
    controller can be acquired. The contentType and template argument allow
    you to restrict the registration to only one content type and choose a
    template other than base_edit, if necessary.
    """
    pfc = getToolByName(context, 'portal_form_controller')
    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'UploadImage',
                      'traverse_to',
                      'string:widget_imagesmanager_upload')

    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'RenameImages',
                      'traverse_to',
                      'string:widget_imagesmanager_rename')

    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'MoveImages',
                      'traverse_to',
                      'string:widget_imagesmanager_move')

    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'DeleteImages',
                      'traverse_to',
                      'string:widget_imagesmanager_delete')


def registerAttachmentsFormControllerActions(context, contentType=None, template='base_edit'):
    """Register the form controller actions necessary for the widget to work.
    This should probably be called from the Install.py script. The parameter
    'context' should be the portal root or another place from which the form
    controller can be acquired. The contentType and template argument allow
    you to restrict the registration to only one content type and choose a
    template other than base_edit, if necessary.
    """
    pfc = getToolByName(context, 'portal_form_controller')
    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'UploadAttachment',
                      'traverse_to',
                      'string:widget_attachmentsmanager_upload')

    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'RenameAttachments',
                      'traverse_to',
                      'string:widget_attachmentsmanager_rename')

    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'MoveAttachments',
                      'traverse_to',
                      'string:widget_attachmentsmanager_move')

    pfc.addFormAction(template,
                      'success',
                      contentType,
                      'DeleteAttachments',
                      'traverse_to',
                      'string:widget_attachmentsmanager_delete')
